import os

from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from tempfile import NamedTemporaryFile


class SmartProxy(models.Model):
    # Smart proxy should have:
    # - smart_proxy hostname / ip address
    # - pxetool_endpoint
    # - pxe server ip address
    hostname = models.CharField(max_length=255)
    username = models.CharField(max_length=64, default='root')
    pxetool_url = models.CharField(max_length=2048, default='http://localhost:8000')
    tftpboot = models.CharField(max_length=255, default='/var/lib/tftpboot')
    port = models.IntegerField(default=22)

    class Meta:
        app_label = "pxetool"

    def __unicode__(self):
        return self.hostname

    def _upload_pxelinux(self):
        from pxetool.tasks import upload_file_task
        task_list = [upload_file_task.si(
            self.pk,
            '/usr/lib/PXELINUX/pxelinux.0',
            os.path.join(self.tftpboot, 'pxelinux.0')
        ), upload_file_task.si(
            self.pk,
            '/usr/lib/syslinux/modules/bios/ldlinux.c32',
            os.path.join(self.tftpboot, 'ldlinux.c32')
        )]
        return task_list

    def _upload_pxelinux_default(self):
        from pxetool.tasks import upload_file_task

        f = NamedTemporaryFile(delete=False)
        f.write("default local\nprompt 0\ntimeout 5\n\nLABEL local\n"
                "  MENU LABEL ^Local Boot\n  MENU default\n  localboot 0\n")
        f.close()
        task_list = [upload_file_task.si(
            self.pk,
            f.name,
            os.path.join(self.tftpboot, 'pxelinux.cfg/default')
        )]
        return task_list

    def init_server(self):
        task = None
        for i in self._upload_pxelinux() + self._upload_pxelinux_default():
            if task:
                task |= i
            else:
                task = i
        return task().task_id


@receiver(post_save, sender=SmartProxy)
def post_save_pxe_server(sender, instance, **kwargs):
    instance.init_server()
