"""
Django settings for pxetool project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""

import django
import os
import re
import ldap
from datetime import timedelta
from django_auth_ldap.config import LDAPSearch, GroupOfNamesType

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'km9@#jr+95=i#xsotkp7l#^agm=1w&_sv0f&hq@jc@60o&&!1w'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'rest_framework',

    'pxetool',
    'pxetool_ui',
    'pxetool_drivers',
]

MIDDLEWARE_CLASSES = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]


ROOT_URLCONF = 'pxetool.urls'

# Template engine for django 1.9
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# Add filters for installation templates for django==1.6 .. 1.9
if django.VERSION < (1, 9):
    from django.template.base import add_to_builtins
    add_to_builtins('pxetool.templatetags.install')
else:
    TEMPLATES[0]['OPTIONS']['builtins'] = ['pxetool.templatetags.install']


WSGI_APPLICATION = 'pxetool.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Password validation
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'

# ========= CUSTOM CONFIGURATION
LOGIN_URL = '/login/'
LOGIN_REDIRECT_URL = '/'

DRIVER_LIST = ['virsh', 'ipmi']

# Compatibility settings
if (1, 6) < django.VERSION < (1, 7):
    INSTALLED_APPS.append('south')
    SOUTH_TESTS_MIGRATE = False

if django.VERSION < (1, 8):
    TEMPLATE_DEBUG = True
    TEMPLATE_LOADERS = (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )

if django.VERSION >= (1, 8):
    MIDDLEWARE_CLASSES.extend([
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    ])

if django.VERSION < (1, 9):
    for driver_name in DRIVER_LIST:
        INSTALLED_APPS.append("pxetool_drivers.{}".format(driver_name))

LOGGING = {
    'version': 1,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
        'pxetool': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
        'pxetool_driver': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
        'pxetool_ui': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
    },
}

# Global variables
AUTH_LDAP_SERVER_URI = "ldap://ldap-msk.msk.mirantis.net"
AUTH_LDAP_BIND_DN = "o=mirantis,dc=mirantis,dc=net"
AUTH_LDAP_USER_DN_TEMPLATE = "uid=%(user)s,ou=people,o=mirantis,dc=mirantis,dc=net"
AUTH_LDAP_BIND_AS_AUTHENTICATING_USER = True
AUTH_LDAP_REQUIRE_GROUP = "cn=mirantis,ou=groups,o=mirantis,dc=mirantis,dc=net"
AUTH_LDAP_GROUP_SEARCH = LDAPSearch(
    "ou=groups,o=mirantis,dc=mirantis,dc=net",
    ldap.SCOPE_SUBTREE, "(objectClass=groupOfNames)")
AUTH_LDAP_GROUP_TYPE = GroupOfNamesType()
AUTH_LDAP_USER_ATTR_MAP = {
    "first_name": "givenName",
    "last_name": "sn",
    "email": "mail"
}

# Use LDAP group membership to calculate group permissions.
AUTH_LDAP_FIND_GROUP_PERMS = True

# Cache group memberships for an hour to minimize LDAP traffic
AUTH_LDAP_CACHE_GROUPS = True
AUTH_LDAP_GROUP_CACHE_TIMEOUT = 3600

AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
]

try:
    import django_auth_ldap
    AUTHENTICATION_BACKENDS.append('django_auth_ldap.backend.LDAPBackend')
except ImportError:
    django_auth_ldap = None

# Base path to store all data
PXETOOL_BASEDIR = '/var/lib/pxetool'
# Place to store profile binary files
PROFILES_FILES = '/var/lib/pxetool'

# CELERY
BROKER_URL = 'redis://localhost:6379'
CELERY_RESULT_BACKEND = 'redis://localhost:6379'
CELERY_ACCEPT_CONTENT = ['application/json']
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'
CELERY_TIMEZONE = 'UTC'

CELERYBEAT_SCHEDULE = {
    'importer': {
        'task': 'fetch_hosts_from_source',
        'schedule': timedelta(seconds=30),
    }
}

REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        # 'rest_framework.authentication.SessionAuthentication',
    )
}

pxetool_cfg = os.getenv('PXETOOL_CFG', False)
if pxetool_cfg and os.path.exists(pxetool_cfg):
    f = open(pxetool_cfg)
    import yaml
    config = yaml.load(f)
    for key, value in config.iteritems():
        globals()[key] = value

COLUMN_HEADERS = [
    'role',
    'status',
    'Additional links',
    'profile'
]


# function name is written using capitals because we need it accessible via
# django.conf.settings module
def COLUMNS(host):
    return [
        host.role if hasattr(host, 'role') else '',
        host.status,
        host.driver.show_links if host.driver else '',
        host.installation.profile_name if host.installation else '',
    ]

MIRROR_REGEX = re.compile(
    '(?P<url_proto>https?|ftps?)://'
    '(?P<url_hostname>(([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])\.)*'
    '([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9]))(?P<url_path_name>/.*)',
    re.IGNORECASE,
)
