from django.conf.urls import include, url
from django.contrib import admin
from django.views.generic import RedirectView
from rest_framework import routers
from pxetool import views_api
from pxetool_ui import views as ui

router = routers.DefaultRouter()
router.register(r'users', views_api.UserViewSet)
router.register(r'groups', views_api.GroupViewSet)
router.register(r'hosts', views_api.HostViewSet)
router.register(r'installations', views_api.InstallationViewSet)
router.register(r'profiles', views_api.ProfileViewSet)

admin.autodiscover()


install_pattern = [
    url(r'^$', ui.InstallView.as_view(), name='install'),
    url(r'^status/(?P<hash>.+)$', ui.StatusUpdateView.as_view(), name='install_status'),
    url(r'^upload/(?P<hash>.+)/$', ui.RecieveLog.as_view(), name='recieve_log'),
    url(r'^upload/(?P<hash>.+)/(?P<id>.+)$', ui.ShowLog.as_view(), name='show_log'),
    url(r'^(?P<hash>.+)/(?P<filename>.+)$', ui.CfgView.as_view(), name='get_install_cfg'),
]

group_pattern = [
    url(r'^$', ui.HostGroupCreateView.as_view(), name='group_create'),
    url(r'^(?P<pk>\d+)/?$', ui.HostGroupUpdate.as_view(), name='group_edit'),
    url(r'^(?P<pk>\d+)/delete', ui.HostGroupDeleteView.as_view(), name='delete_group'),
]

host_pattern = [
    url(r'^$', ui.CreateHostView.as_view(), name='host_create'),
    url(r'^(?P<pk>\d+)?$', ui.HostDetail.as_view()),
    url(r'^delete', ui.DeleteHostView.as_view(), name='host_delete', kwargs={'action': 'delete'}),
    url(r'^abort', ui.AbortInstallation.as_view(), name='abort_install'),  # ToDo: move to installations, or keep here?
    url(r'^(?P<pk>\d+)/edit$', ui.UpdateHostView.as_view(), name='host_edit'),
]

smart_proxy_pattern = [
    url(r'^$', ui.SmartProxyCreate.as_view(), name='smart_proxy_create'),
    url(r'^(?P<pk>\d+)/?$', ui.SmartProxyUpdate.as_view(), name='smart_proxy_update'),
    url(r'^(?P<pk>\d+)/delete/?$', ui.SmartProxyDelete.as_view(), name='smart_proxy_delete'),
]

source_pattern = [
    url(r'^$', ui.HostSourceCreate.as_view(), name='source_create'),
    url(r'^(?P<pk>\d+)$', ui.HostSourceUpdate.as_view(), name='source_update'),
    url(r'^(?P<pk>\d+)/delete/$', ui.HostSourceDelete.as_view(), name='source_delete'),
    url(r'^(?P<pk>\d+)/enable/$', ui.HostSourceEnable.as_view(), name='source_enable'),
]

profile_pattern = [
    url(r'^$', ui.ProfileImport.as_view(), name='profile_import'),
    url(r'^(?P<pk>\d+)', ui.ProfileImport.as_view(), name='profile_import'),
]

puppet_master = [
    url(r'^$', ui.PuppetMasterCreate.as_view(), name='puppet_master_create'),
    url(r'^(?P<pk>\d+)$', ui.PuppetMasterUpdate.as_view(), name='puppet_master_update'),
    url(r'^(?P<pk>\d+)/delete/$', ui.PuppetMasterDelete.as_view(), name='puppet_master_delete'),
]

urlpatterns = [
    url(r'^api/v1/', include(router.urls)),
    url(r'^favicon\.ico$', RedirectView.as_view(url='/static/favicon.ico')),
    url(r'^$', ui.HostGroupView.as_view(), name='index'),
    url(r'^', include('django.contrib.auth.urls')),
    url(r'^category/(?P<active_group>.*)$', ui.HostGroupView.as_view(), name='category'),
    url(r'^docs/(?P<page>.*)', ui.DocsView.as_view(), name='docs'),
    url(r'^install/', include(install_pattern)),
    url(r'^admin/', include(admin.site.urls)),

    # modules
    url(r'^hosts$', ui.HostGroupView.as_view()),  # same as index
    url(r'^host/', include(host_pattern)),
    url(r'^groups', ui.HostGroupView.as_view()),  # same as index
    url(r'^group/', include(group_pattern)),
    url(r'^profiles$', ui.ProfileList.as_view(), name='profile_list'),
    url(r'^profile/', include(profile_pattern)),
    url(r'^smart-proxies$', ui.SmartProxyList.as_view(), name='smart_proxy_list'),
    url(r'^smart-proxy/', include(smart_proxy_pattern)),
    url(r'^sources$', ui.HostSourceList.as_view(), name='sources_list'),  # ToDo: rename to source_list
    url(r'^source/', include(source_pattern)),
    url(r'^puppet-masters$', ui.PuppetMasterList.as_view(), name='puppet_master_list'),
    url(r'^puppet-master/', include(puppet_master)),
]
