from django.contrib.auth.models import User, Group
from pxetool.models import Host, Installation, Profile
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from pxetool.serializers import (
    UserSerializer,
    GroupSerializer,
    HostSerializer,
    InstallationSerializer,
    ProfileSerializer,
)


class AuthMixin(object):
    permission_classes = (IsAuthenticated, )


class UserViewSet(AuthMixin, viewsets.ModelViewSet):
    queryset = User.objects.all().order_by('-date_joined')
    serializer_class = UserSerializer


class GroupViewSet(AuthMixin, viewsets.ModelViewSet):
    queryset = Group.objects.all()
    serializer_class = GroupSerializer


class HostViewSet(AuthMixin, viewsets.ModelViewSet):
    queryset = Host.objects.all()
    serializer_class = HostSerializer


class ProfileViewSet(AuthMixin, viewsets.ModelViewSet):
    queryset = Profile.objects.all()
    serializer_class = ProfileSerializer


class InstallationViewSet(AuthMixin, viewsets.ModelViewSet):
    queryset = Installation.objects.all()
    serializer_class = InstallationSerializer
