from django import forms
import subprocess
import re
from .base_driver import BaseDriver

from logging import Logger

logger = Logger(__name__)


class IpmiForm(forms.Form):
    ipmi_hostname = forms.CharField(max_length=255, initial='')
    ipmi_username = forms.CharField(max_length=255, initial='')
    ipmi_password = forms.CharField(max_length=255, initial='')


def _run(command):
    output = ''
    try:
        logger.debug("Run: {}".format(command))
        output = subprocess.check_output(
            command.split(' '), stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        logger.error("Failed {0}: Exitcode: {1}\n Output: {2}"
                     .format(command, e.returncode, e.output))
    return output


class Ipmi(BaseDriver):
    def __init__(self, *args, **kwargs):
        (self.host,) = args
        self.hostname = kwargs.get('ipmi_hostname', '')
        self.username = kwargs.get('ipmi_username', '')
        self.password = kwargs.get('ipmi_password', '')

    def power_cycle(self):
        ipmi = "ipmitool -I lanplus -L operator -H{} -U{} -P{} chassis" \
            .format(self.hostname, self.username, self.password)
        _run('{} bootdev pxe'.format(ipmi))
        status = _run('{} power status'.format(ipmi))
        if re.match("Chassis Power is on", status):
            command = '{} power reset'.format(ipmi)
        else:
            command = '{} power on'.format(ipmi)
        _run(command)

    @property
    def show_links(self):
        ipmi = ('<form action="http://{hostname}/cgi/login.cgi" '
                'method="post" target="_blank">'
                '<input type="hidden" name="name" value="{username}" />'
                '<input type="hidden" name="pwd" value="{password}" />'
                '<input type="submit" value="IPMI Login"/></form>').format(
            hostname=self.hostname.strip(' '),
            username=self.username,
            password=self.password,
        )
        return '\n'.join([ipmi])

    @staticmethod
    def get_form(prefix='ipmi', **kwargs):
        return IpmiForm(prefix=prefix, **kwargs)
