import os
import pwd
import types

from pxetool.models import Host, HostGroup, SmartProxy
from .models import DriverManager
from .base_driver import BaseDriver
from django.test import TestCase
from django.conf import settings

driver_list = settings.DRIVER_LIST


class TestDriverManager(TestCase):
    hosts = {}

    def setUp(self):
        smart_proxy = SmartProxy.objects.create(
            hostname='localhost',
            username=pwd.getpwuid(os.getuid()).pw_name,
        )
        hg = HostGroup.objects.create(name='hg', smart_proxy=smart_proxy)
        self.hosts['ipmi'] = Host.objects.create(
            hostname='ipmi',
            mac='01-02-03-04-05-06',
            group=hg,
        )
        DriverManager.objects.create(driver_name='ipmi', host=self.hosts['ipmi'])
        self.hosts['virsh'] = Host.objects.create(
            hostname='virsh',
            mac='01-02-03-04-05-07',
            group=hg,

        )
        DriverManager.objects.create(
            driver_name='virsh',
            params={'conn_string': 'qemu+ssh://ya.ru/system'},
            host=self.hosts['virsh'],
        )
        self.hosts['powerless'] = Host.objects.create(
            hostname='powerless',
            mac='01-02-03-04-05-08',
            group=hg,
        )
        DriverManager.objects.create(
            driver_name='powerless',
            host=self.hosts['powerless']
        )

    def testValidNames(self):
        for k, v in self.hosts.iteritems():
            self.assertEqual(k, str(v.driver))

    def testDriverSubclassBaseDriver(self):
        for driver_name in driver_list:
            self.hosts[driver_name].driver._init_driver()
            self.assertIsInstance(self.hosts[driver_name].driver.driver, BaseDriver)

    def testDriverFunctions(self):
        allowed_functions = [
            'power_cycle',
            'power_on',
            'power_off',
        ]
        for k, v in self.hosts.iteritems():
            for function in allowed_functions:
                self.assertIsInstance(
                    getattr(v.driver, function),
                    (types.FunctionType, types.MethodType)
                )

    def testDriverProperties(self):
        allowed_properties = [
            'power_state',
            'screenshot',
            'show_links',
        ]
        for name in driver_list:
            self.hosts[name].driver._init_driver()
            driver = self.hosts[name].driver.driver
            for property_name in allowed_properties:
                self.assertIsInstance(
                    getattr(driver.__class__, property_name),
                    property
                )

    def testDriverForm(self):
        pass
