from django import template
from django.conf import settings
from collections import OrderedDict

from pxetool.models import Host
from pxetool_drivers.models import DriverManager
from pxetool.models import form_profile_host

from pxetool_ui.forms import HostForm
from pxetool_drivers.forms import DriverForm

import logging

logger = logging.getLogger()
register = template.Library()


@register.inclusion_tag('hosts/host_form.html')
def add_host_form(form=HostForm(), host=None):
    power_forms = OrderedDict()
    driver_form = None
    power_forms['powerless'] = {'content': 'No power control', 'active': ''}
    for driver_name in settings.DRIVER_LIST:
        if host and host.driver.driver_name == driver_name and driver_name != 'powerless':
            power_forms[driver_name] = {
                'content': '',
                'form': host.driver.get_form(),
                'active': 'active'
            }
            driver_form = DriverForm(initial={'driver_name': driver_name})
        else:
            power_forms[driver_name] = {'content': '',
                                        'form': DriverManager.get_form(driver_name),
                                        'active': ''}
    if not driver_form:
        driver_form = DriverForm(initial={'driver_name': 'powerless'})
        power_forms['powerless']['active'] = 'active'
    if host:
        profile_form = form_profile_host(data=host.params)
    else:
        profile_form = form_profile_host()
    return {'host_form': form,
            'profile_form': profile_form,
            'power_form': driver_form,
            'power_forms': power_forms}


@register.inclusion_tag('hosts/host_list.html')
def host_list(group_id):
    hosts = Host.objects.filter(group_id=group_id).select_related('driver', 'installation', 'profile', 'group')
    column_headers = [
        'status',
        'Additional links',
        'profile',
    ]
    if hasattr(settings, 'COLUMN_HEADERS') and hasattr(settings, 'COLUMNS'):
        column_headers = settings.COLUMN_HEADERS
        columns = settings.COLUMNS
    else:
        if hasattr(settings, 'COLUMN_HEADERS') or hasattr(settings, 'COLUMNS'):
            logger.warn('settings has incorrect COLUMNS or COLUMN_HEADERS, failing back to default')

        def columns(host):
            return [
                host.status,
                host.driver.show_links if host.driver else None,
                host.installation.profile_name if host.installation else '',
            ]
    for host in hosts:
        host.columns = columns(host)
    return {'hosts': hosts, 'group_id': group_id, 'column_headers': column_headers}
